package com.chelvc.framework.security.context;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Sets;
import lombok.NonNull;
import org.springframework.security.oauth2.jwt.Jwt;

/**
 * 认证上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class AuthorizeContextHolder {
    /**
     * 应用范围标识
     */
    public static final String SCOPE = "scope";

    /**
     * 客户端登陆标识
     */
    public static final String CLIENT = "client";

    /**
     * 手机号码标识
     */
    public static final String MOBILE = "mobile";

    /**
     * 用户账号标识
     */
    public static final String USERNAME = "user_name";

    /**
     * 主体创建时间戳
     */
    public static final String CREATING = "creating";

    /**
     * 主体注册时间戳
     */
    public static final String REGISTERING = "registering";

    /**
     * 用户权限标识
     */
    public static final String AUTHORITIES = "authorities";

    private AuthorizeContextHolder() {
    }

    /**
     * 构建访问令牌标识
     *
     * @param id 主体标识
     * @return 令牌标识
     */
    public static String key(@NonNull Object id) {
        return "token:" + id;
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(USERNAME), Long::parseLong);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(SCOPE), (String) null);
    }

    /**
     * 是否是客户端登陆
     *
     * @param jwt Jwt对象
     * @return true/false
     */
    public static boolean isClient(Jwt jwt) {
        return jwt != null && Boolean.TRUE.equals(jwt.getClaimAsBoolean(CLIENT));
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }
}
